@@ -1,26 +1,25 @@
 # Statistics Menu dialogs
 
-# last modified 3 Jan 05 by J. Fox
+# last modified 13 Mar 05 by J. Fox
 
     # Dimensional-analysis menu
     
 Reliability <- function(){
-    if(!checkActiveDataSet()) return()
-    if (!checkNumeric(3)) return()
+##    if(!checkActiveDataSet()) return()
+##    if (!checkNumeric(3)) return()
     initializeDialog(title="Scale Reliability")
-    xBox <- variableListBox(top, .numeric, selectmode="multiple", title="Variables (pick three or more)")
+    xBox <- variableListBox(top, Numeric(), selectmode="multiple", title="Variables (pick three or more)")
     onOK <- function(){
         x <- getSelection(xBox)
+        closeDialog()
         if (3 > length(x)) {
             errorCondition(recall=Reliability, message="Fewer than 3 variables selected.")
             return()
             }
-        if (.grab.focus) tkgrab.release(top)
-        tkdestroy(top)
         x <- paste('"', x, '"', sep="")
-        doItAndPrint(paste("reliability(cov(", .activeDataSet, "[,c(", paste(x, collapse=","),
+        doItAndPrint(paste("reliability(cov(", ActiveDataSet(), "[,c(", paste(x, collapse=","),
             ')], use="complete.obs"))', sep=""))
-        tkfocus(.commander)
+        tkfocus(CommanderWindow())
         }
     OKCancelHelp(helpSubject="reliability")
     tkgrid(getFrame(xBox), sticky="nw")
@@ -29,28 +28,29 @@ Reliability <- function(){
     }
 
 principalComponents <- function(){
-    if(!checkActiveDataSet()) return()
-    if(!checkNumeric(2)) return()
+##    if(!checkActiveDataSet()) return()
+##    if(!checkNumeric(2)) return()
     initializeDialog(title="Principal Components Analysis")
-    xBox <- variableListBox(top, .numeric, selectmode="multiple", title="Variables (pick two or more)")
+    xBox <- variableListBox(top, Numeric(), selectmode="multiple", title="Variables (pick two or more)")
     subsetBox()
     checkBoxes(frame="optionsFrame", boxes=c("correlations", "screeplot", "addPC"), initialValues=c("1", "0", "0"),
         labels=c("Analyze correlation matrix", "Screeplot", "Add principal components to data set"))
     onOK <- function(){
+        putRcmdr("ncomponents", 0)
         x <- getSelection(xBox)
         nvar <- length(x)
         correlations <- tclvalue(correlationsVariable)
         subset <- tclvalue(subsetVariable)
         screeplot <- tclvalue(screeplotVariable)
         addPC <- tclvalue(addPCVariable)
+        closeDialog()
         if (2 > length(x)) {
             errorCondition(recall=principalComponents, message="Fewer than 2 variables selected.")
             return()
             }
-        if (.grab.focus) tkgrab.release(top)
-        tkdestroy(top)
         subset <- if (trim.blanks(subset) == "<all valid cases>") "" else paste(", subset=", subset, sep="")
         correlations <- if (correlations == "1") "TRUE" else "FALSE"
+        .activeDataSet <- ActiveDataSet()
         command <- paste("princomp(~", paste(x, collapse="+"), ", cor=", correlations,
             ", data=", .activeDataSet, subset, ")", sep="")
         assign(".PC", justDoIt(command), envir=.GlobalEnv)
@@ -70,31 +70,29 @@ principalComponents <- function(){
                 resolution=1, orient="horizontal")
             componentsShow <- tklabel(sliderFrame, textvariable=sliderValue, width=2, justify="right")
             onOKsub <- function() {
-                if (.grab.focus) tkgrab.release(subdialog)
-                tkdestroy(subdialog)
-                assign(".ncomponents", as.numeric(tclvalue(sliderValue)), envir=.GlobalEnv)
+                closeDialog(subdialog)
+                putRcmdr("ncomponents", as.numeric(tclvalue(sliderValue)))
                     }
             subOKCancelHelp()
             tkgrid(componentsSlider, componentsShow, sticky="nw")
             tkgrid(sliderFrame, sticky="w")
             tkgrid(subButtonsFrame, sticky="w")
             dialogSuffix(subdialog, onOK=onOKsub, rows=2, columns=1, focus=subdialog)
-            if (exists(".ncomponents", envir=.GlobalEnv)){
-                for(i in 1:.ncomponents){
+            if ((ncomponents <- getRcmdr("ncomponents")) > 0){
+                for(i in 1:ncomponents){
                     var <- paste("PC", i, sep="")
-                    if (is.element(var, .variables)) {
+                    if (is.element(var, Variables())) {
                         if ("no" == tclvalue(checkReplace(var))) next
                         }
                     justDoIt(paste(.activeDataSet, "$PC", i, " <- .PC$scores[,", i, "]", sep=""))
                     logger(paste(.activeDataSet, "$PC", i, " <- .PC$scores[,", i, "]", sep=""))
                     }
                 activeDataSet(.activeDataSet)
-                remove(.ncomponents, envir=.GlobalEnv)
                 }
             }
         remove(.PC, envir=.GlobalEnv)   
         logger("remove(.PC)")
-        tkfocus(.commander)
+        tkfocus(CommanderWindow())
         }
     OKCancelHelp(helpSubject="princomp")
     tkgrid(getFrame(xBox), sticky="nw")
@@ -105,10 +103,10 @@ principalComponents <- function(){
     }
 
 factorAnalysis <- function(){
-    if(!checkActiveDataSet()) return()
-    if(!checkNumeric(3)) return()
+##    if(!checkActiveDataSet()) return()
+##    if(!checkNumeric(3)) return()
     initializeDialog(title="Factor Analysis")
-    xBox <- variableListBox(top, .numeric, selectmode="multiple", title="Variables (pick three or more)")
+    xBox <- variableListBox(top, Numeric(), selectmode="multiple", title="Variables (pick three or more)")
     subsetBox()
     optionsFrame <- tkframe(top)
     checkFrame <- tkframe(top)
@@ -124,6 +122,7 @@ factorAnalysis <- function(){
         subset <- tclvalue(subsetVariable)
         rotation <- tclvalue(rotationVariable)
         scores <- tclvalue(scoresVariable)
+        closeDialog()
         if (3 > length(x)) {
             errorCondition(recall=factorAnalysis, message="Fewer than 3 variables selected.")
             return()
@@ -138,31 +137,25 @@ factorAnalysis <- function(){
             resolution=1, orient="horizontal")
         componentsShow <- tklabel(sliderFrame, textvariable=sliderValue, width=2, justify="right")
         onOKsub <- function() {
-            if (.grab.focus) tkgrab.release(subdialog)
-            tkdestroy(subdialog)
-            assign(".nfactors", as.numeric(tclvalue(sliderValue)), envir=.GlobalEnv)
+            closeDialog(subdialog)
+            putRcmdr("nfactors", as.numeric(tclvalue(sliderValue)))
                 }
         subOKCancelHelp()
         tkgrid(componentsSlider, componentsShow, sticky="nw")
         tkgrid(sliderFrame, sticky="w")
         tkgrid(subButtonsFrame, sticky="w")
         dialogSuffix(subdialog, onOK=onOKsub, rows=2, columns=1, focus=subdialog)
-        if (.grab.focus) tkgrab.release(top)
-        tkdestroy(top)
-        if (!exists(".nfactors", envir=.GlobalEnv)){
-            tkfocus(.commander)
-            return()
-            }
         subset <- if (trim.blanks(subset) == "<all valid cases>") "" else paste(", subset=", subset, sep="")
-        command <- paste("factanal(~", paste(x, collapse="+"), ", factors=", .nfactors, ', rotation="', rotation,
+        .activeDataSet <- ActiveDataSet()
+        command <- paste("factanal(~", paste(x, collapse="+"), ", factors=", getRcmdr("nfactors"), ', rotation="', rotation,
             '", scores="', scores, '", data=', .activeDataSet, subset, ")", sep="")
         assign(".FA", justDoIt(command), envir=.GlobalEnv)
         logger(paste(".FA <- ", command, sep=""))
         doItAndPrint(".FA")
         if (scores != "none") {
-            for(i in 1:nfactor){
+            for(i in 1:getRcmdr("nfactors")){
                 var <- paste("F", i, sep="")
-                if (is.element(var, .variables)) {
+                if (is.element(var, Variables())) {
                     if ("no" == tclvalue(checkReplace(var))) next
                     }
                 justDoIt(paste(.activeDataSet, "$F", i, " <- .FA$scores[,", i, "]", sep=""))
@@ -171,8 +164,8 @@ factorAnalysis <- function(){
             activeDataSet(.activeDataSet)
             }
         logger("remove(.FA)")
-        remove(.FA, .nfactors, envir=.GlobalEnv)
-        tkfocus(.commander)
+        remove(.FA, envir=.GlobalEnv)
+        tkfocus(CommanderWindow())
         }
     OKCancelHelp(helpSubject="factanal")
     tkgrid(getFrame(xBox), sticky="nw")
